/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gui;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.CollectionDesignManager;
import org.greenstone.gatherer.msm.ElementWrapper;
/** This serves as the Collection configuration pane and came in two flavors.<BR>
 * The lower tech version has now been made redundant.<BR>
 * The higher tech, and finally implemented, version uses a tree to allow a user to navigate through the various regions of the config file and further allows them to interactively design and modify such gsdl elements as classifiers and plugins.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.2
 */
public class GConfigPane 
    extends JPanel {
    /** The collection manager is responsible for parsing in, and allowing the editing of, a single collections configuration file. */
    private CollectionDesignManager cdm = null;
    /** The constructor. */
    public GConfigPane() {
    }
    /** This method is called whenever the state of the current collection
     * changes.
     * @param ready <i>true</i> if a collection is loaded and ready, <i>false</i> otherwise.
     */
    public void collectionChanged(boolean ready) {
	///atherer.println("CollectionChanged(" + ready + ")");
	if(ready) {
	    // Retrieve the new config manager.
	    cdm = Gatherer.c_man.getCollection().cdm;
	    // Display it.
	    cdm.display(this);
	}
	else {
	    if(cdm != null) {
		cdm.destroy();
		cdm = null;
	    }
	}
    }
    /** Called to cause the components to lay themselves out and be displayed.
     */
    public void display() {
	// Layout
	this.setLayout(new BorderLayout());
    }
    public void gainFocus() {
	if(cdm == null && Gatherer.c_man.ready()) {
	    // Retrieve the new config manager.
	    cdm = Gatherer.c_man.getCollection().cdm;
	    // Display it.
	    cdm.display(this);
	}
	if(cdm != null) {
	    cdm.gainFocus();
	}
    }

    public ArrayList getLanguageCodes() {
	return cdm.language_manager.getLanguageCodes();
    }

    public Rectangle setSelectedElement(ElementWrapper element) {
	return cdm.setSelectedElement(element.getName());
    }
    public void saveConfiguration() {
	if(cdm != null) {
	    cdm.save();
	}
    }
}
