#!/usr/bin/perl -w

###########################################################################
#
# pluginfo.pl --
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 1999 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

# This program will print info about a plugin

# @revision 09/05/02 Added XML usage information flag - John Thompson

BEGIN {
    die "GSDLHOME not set\n" unless defined $ENV{'GSDLHOME'};
    die "GSDLOS not set\n" unless defined $ENV{'GSDLOS'};
    unshift (@INC, "$ENV{'GSDLHOME'}/perllib");
    unshift (@INC, "$ENV{'GSDLHOME'}/perllib/plugins");
}

use plugin;
use util;
use parsargv;
use gsprintf;
use printusage;


my $arguments =
    [ { 'name' => "collect",
	'desc' => "{pluginfo.collect}",
	'type' => "string",
	'reqd' => "no" },
      { 'name' => "xml",
	'desc' => "{scripts.xml}",
	'type' => "flag",
	'reqd' => "no" },
      { 'name' => "language",
	'desc' => "{scripts.language}",
	'type' => "string",
	'reqd' => "no" } ];

my $options = { 'name' => "pluginfo.pl",
		'desc' => "{pluginfo.desc}",
		'args' => $arguments };


sub lookup_string
{
    return &gsprintf::lookup_string($language, shift(@_));
}


sub print_txt_usage
{
    local $language = shift(@_);

    local $programname = $options->{'name'};
    local $programargs = $options->{'args'};

    # Find the length of the longest option string
    local $descoffset = 0;
    if (defined($programargs)) {
	$descoffset = &PrintUsage::find_longest_option_string($programargs);
    }

    # Produce the usage information using the data structure above
    print STDERR $programname . ": " . &lookup_string($options->{'desc'}) . "\n\n";
    print STDERR " " . &lookup_string("{common.usage}") . ": $programname";
    print STDERR " " . &lookup_string("{pluginfo.params}") . "\n\n";

    # Display the program options, if there are some
    if (defined($programargs)) {
	# Calculate the column offset of the option descriptions
	local $optiondescoffset = $descoffset + 2;  # 2 spaces between options & descriptions

	print STDERR " " . &lookup_string("{common.options}") . ":\n";

	# Display the program options
	&PrintUsage::print_options_txt($language, $programargs, $optiondescoffset);
    }
}


#  sub print_usage {
#      print STDERR "\n";
#      print STDERR "pluginfo.pl: Prints information about a plugin.\n\n";
#      print STDERR " usage: $0 [options] plugin\n\n";
#      print STDERR " options:\n";
#      print STDERR "  -collect collection-name  Giving a collection name will make pluginfo.pl\n";
#      print STDERR "                            look in collect/collection-name/perllib/plugins\n";
#      print STDERR "                            first. If the plugin is not found there it will\n";
#      print STDERR "                            look in the general perllib/plugins directory.\n\n";
#      print STDERR "  -xml                      Produces the information in an xml form, without\n";
#      print STDERR "                            'pretty' comments but with much more detail.\n\n";
#      print STDERR "  -language language-code   Language to display plugin options in (eg. 'en_US'\n";
#      print STDERR "                            specifies American English). Requires translations\n";
#      print STDERR "                            of the option descriptions to exist in the\n";
#      print STDERR "                            perllib/strings_language-code.rb file.\n";
#  }

sub main {
    my $collect = "";
    my $xml = 0;
    my $language = "";  # Will display in the default language if not set

    if (!parsargv::parse(\@ARGV,
			 q^collect/.*/^, \$collect,
			 q^xml^, \$xml,
			 q^language/.*/^, \$language))
    {
	&print_txt_usage($language);
	die "\n";
    }

    my $plugin = shift (@ARGV);
    if (!defined $plugin || $plugin eq "") {
	print STDERR &lookup_string("{pluginfo.no_plugin_name}") . "\n\n";
	&print_txt_usage($language);
	die "\n";
    }

    if ($collect ne "") {
	$ENV{'GSDLCOLLECTDIR'} = &util::filename_cat ($ENV{'GSDLHOME'}, "collect", $collect);
    } else {
	$ENV{'GSDLCOLLECTDIR'} = $ENV{'GSDLHOME'};
    }

    my $pluginfo = &plugin::load_plugins ([[$plugin]]);
    my $plugobj = shift @$pluginfo;

    if ($xml) {
        $plugobj->print_xml_usage($language);
    }
    else {
	print STDERR "\n$plugin " . &lookup_string("{pluginfo.info}") . ":\n\n";
	print STDERR &lookup_string("{pluginfo.passing_options}") . "\n\n";
	print STDERR &lookup_string("{pluginfo.option_types}") . ":\n\n";
	print STDERR &lookup_string("{pluginfo.specific_options}") . "\n\n";
	print STDERR &lookup_string("{pluginfo.general_options}") . "\n\n";

	$plugobj->print_txt_usage($language);

# 	print STDERR " The default process_exp for $plugin is: ";
# 	print STDERR "\"" . $plugobj->get_default_process_exp() . "\"\n\n";

# 	print STDERR " The default block_exp for $plugin is: ";
# 	print STDERR "\"" . $plugobj->get_default_block_exp() . "\"\n\n";
    }
}


&main ();
